/*

	Framework - Window.h
	File Version 1.0.000

*/

#ifndef _WINDOW_H_
#define _WINDOW_H_

#include <windows.h>
#include "xchar.h"

namespace Framework
{

	class CWindow
	{
	public:
							CWindow();
		virtual				~CWindow();
		static void			Initialize();
		static void			Release();
		static long	WINAPI	WndProc(HWND, unsigned int, WPARAM, LPARAM);
		static void			StdMsgLoop(CWindow*);
		static int			MessageBoxFormat(HWND, unsigned int, const xchar*, const xchar*, ...);
		void				SetClassPtr();
		void				ClearClassPtr();
		unsigned int		DoesWindowClassExist(xchar*);

		//Helpers
		void				Create(unsigned long, const xchar*, const xchar*, unsigned long, RECT*, HWND, void*);  
		unsigned int		Destroy();
		unsigned int		Show(int);
		unsigned int		Enable(unsigned int);
		bool				IsWindow();
		bool				IsVisible();
		int					GetText(xchar*, int);
		unsigned int		GetTextLength();
		unsigned int		SetText(const xchar*);
		unsigned int		SetTextA(const char*);
		void				SetFont(HFONT);
		void				SetMenu(HMENU);
		void				SetIcon(unsigned int, HICON);
		void				SetFocus();
		void				SetSize(unsigned int, unsigned int);
		void				SetPosition(unsigned int, unsigned int);
		void				Center(HWND = 0);
		void				GetClientRect(RECT*);
		HWND				GetParent();
		void				Redraw();
		void				ModifyStyleOr(unsigned long);

	protected:
		virtual long		OnCommand(unsigned short, unsigned short, HWND);
		virtual long		OnSysCommand(unsigned int, LPARAM);
		virtual long		OnNotify(WPARAM, NMHDR*);
		virtual long		OnWndProc(unsigned int, WPARAM, LPARAM);
		virtual long		OnSize(unsigned int, unsigned int, unsigned int);
		virtual long		OnLeftButtonUp(unsigned int, unsigned int);
		virtual long		OnLeftButtonDown(unsigned int, unsigned int);
		virtual long		OnLeftButtonDblClk(unsigned int, unsigned int);
		virtual long		OnRightButtonUp(unsigned int, unsigned int);
		virtual long		OnMouseMove(WPARAM, unsigned int, unsigned int);
		virtual long		OnMouseWheel(short);
		virtual long		OnVScroll(unsigned int, unsigned int);
		virtual long		OnHScroll(unsigned int, unsigned int);
		virtual long		OnSysKeyDown(unsigned int);
		virtual long		OnKeyDown(unsigned int);
		virtual long		OnKeyUp(unsigned int);
		virtual long		OnDestroy();
		virtual long		OnCtlColorStatic(HDC, HWND);
		virtual long		OnActivate(unsigned int, bool, HWND);
		virtual long		OnActivateApp(bool, unsigned long);

		//Add parameters for these
		virtual long		OnPaint();
		virtual long		OnMouseLeave();
		virtual long		OnEraseBkgnd();
		virtual long		OnTimer();
		virtual long		OnSetFocus();
		virtual long		OnKillFocus();

	public:
		HWND				m_hWnd;
		unsigned int		m_nNoCallDef;
		static ATOM			m_nAtom;
	};

};

#endif